/* Unterprogramme fr SPI_LCD
 * Datei: spi_lcd.c
 * Info: Unterprogrammgruppe SPI_LCD
 * Autor: Dirk ????
 * Gendert:Pirkl Max
 * Datum: ????
*/

#include "mcu.h"
#include "spi_lcd.h"

static volatile unsigned char 	rs;

//Funktion zum senden ber SPI
void SPI_sende (unsigned char aData) 
{
	SPDR = aData;										
	while(!(SPSR & (1<<SPIF)));						
}

//Funktion zum senden von kommandos
void writecmd (unsigned char cmd)
{
	unsigned char lower;
	unsigned char upper;
	unsigned char startbyte;
	lower = cmd % 16; //Die unteren 4 bit isolieren
	upper = cmd / 16; //Die oberen 4 bit isolieren
	if (rs != 0) 
	{
		startbyte = 0x1F;
		SPI_sende (startbyte);
	}
	SPI_sende (lower); //senden der unteren 4 bits
	SPI_sende (upper); //senden der oberen 4 bits
	rs = 0;
	_delay_us(45); //Warten bis befehl ausgefhrt
}

//Funktion zum senden von Zeichen
void writedat (unsigned char dat)
{
	unsigned char lower;
	unsigned char upper;
	unsigned char startbyte;
	lower = dat % 16; //Die unteren 4 bit isolieren
	upper = dat / 16; //Die oberen 4 bit isolieren
	if (rs !=1)
	{
		startbyte = 0x5F;
		SPI_sende (startbyte);
	}
	SPI_sende (lower); //senden der unteren 4 bits
	SPI_sende (upper); //senden der oberen 4 bits
	rs = 1;
	_delay_us(55); //Warten bis befehl ausgefhrt
}

void lcd_init(void)	
{
	DDRB = (1<<DDB0)|(1<<DDB1)|(1<<DDB2);	//SPI Pins als ausgang
	//SPI einstellen:SPI einschalten(SPE),Master modus(MSTR),SPI Taktrate teiler(kleiner 270kHz),Data order(DORD) LSB zuerst
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<DORD);
	_delay_ms(40); //Warten um sicherzustellen das Displayspannung ok
	rs = 1;
	writecmd(0x34); //8-Bit Datenlnge, extension Bit RE=1
	writecmd(0x09); //4 Zeilen Modus
	writecmd(0x30); //8-Bit Datenlnge, extension Bit RE=0
	writecmd(0x0F); //Display ein, Cursor ein, Cursor blinken
	//_delay_ms(2);
	writecmd(Clear_Display); //Display lschen, Cursor auf 1.Spalte von 1.Zeile
	_delay_ms(2);
	writecmd(0x06); //Cursor Auto-increment
}

//Funktion zum senden eines Zeichens
void lcd_chr(unsigned char c)
{
writedat(c);
}

//Funktion zum senden eines Strings
void lcd_str(const char *s)
{
    register unsigned char c;
    while ( (c = *s++) )
    {
     lcd_chr(c);
    }
}

//Funktion zum lschen des Displays
void lcd_clear(void)
{
	writecmd(Clear_Display);
	_delay_ms(2);
}

//Funktion zum rcksprung an home position
void lcd_home(void)
{
	writecmd(Home_Display);
	_delay_ms(2);
}

//Funktion zum ausschlten des cursors
void lcd_cursoroff(void)
{
	writecmd(Cursoroff_Display);
}

//Funktion zum einschalten des cursors
void lcd_cursoron(void)
{
	writecmd(Cursoron_Display);
}

//Funktion zum springen an eine belibige position
void lcd_goto(unsigned char zeile, unsigned char spalte)
{
	unsigned char position = 0x00;
	switch(zeile){
		case 1:{
			position = 0X00; //Anpassen fals andere displaygrse
			break;
		}
		case 2:{
			position = 0X20; //Anpassen fals andere displaygrse
			break;
		}
		case 3:{
			position = 0X40; //Anpassen fals andere displaygrse
			break;
		}
		case 4:{
			position = 0X60; //Anpassen fals andere displaygrse
			break;
		}
	}     
	position = (position+0x80);	 //hinzufgen des befehl-bits
	position = (position+spalte); //zusammenfhren von zeile und spalte
	writecmd(position);
}
